package com.yovole.sdk;

import com.yovole.sdk.common.auth.ICloudCredentials;
import com.yovole.sdk.common.common.Endpoint;
import com.yovole.sdk.common.exception.SdkBaseException;
import com.yovole.sdk.common.model.BaseModel;
import com.yovole.sdk.common.response.BaseAPIResponse;
import com.yovole.sdk.common.response.ErrorAPIResponse;
import com.yovole.sdk.common.sign.Signers;
import com.yovole.sdk.common.sign.impl.HmacSHA1Signer;
import com.yovole.sdk.common.utils.DateUtils;
import com.yovole.sdk.common.utils.JsonUtils;
import com.yovole.sdk.support.Endpoints;
import org.apache.http.Header;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.StatusLine;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpRequestBase;
import org.apache.http.conn.ssl.NoopHostnameVerifier;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.util.EntityUtils;
import org.apache.log4j.*;

import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import java.io.IOException;
import java.security.cert.X509Certificate;
import java.util.*;

/**
 * Author : dujie
 * Created : 2018/10/16 16:46
 */
public class ApiSdkClient {
    private ICloudCredentials credentials;
    private ApiSdkProfile profile;
    protected Logger logger;

    public ApiSdkClient(ICloudCredentials credentials, ApiSdkProfile profile) {
        this.credentials = credentials;
        this.profile = profile;
        if(this.credentials == null) throw new NullPointerException("credentials is need");
        if(profile == null) {
            this.profile = this.getDefaultApiSdkProfile();
        }

        this.logger = Logger.getLogger(getClass());
    }

    protected ApiSdkProfile getDefaultApiSdkProfile() {
        ApiSdkProfile profile = new ApiSdkProfile();
        profile.setApiVersion("v1");
        profile.setSdkVersion("v1");
        profile.setSignerName(HmacSHA1Signer.NAME);
        profile.setEndpoint(Endpoints.getGlobalEndpoint());
        return profile;
    }

    protected ICloudCredentials getCredentials() {
        return credentials;
    }

    public ApiSdkProfile getProfile() {
        return profile;
    }

    protected <ReqModal extends BaseModel, ResModal extends BaseModel, Res extends BaseAPIResponse<ResModal>> Res executePost(String uri, ReqModal requestModal, Class<Res> responseClass) throws SdkBaseException {
        try {
            String url = this.resolveApiUrl(uri);

            SSLContext sslContext = SSLContext.getInstance("TLS");
            sslContext.init(null, new TrustManager[]{new X509TrustManager() {
                @Override
                public void checkClientTrusted(X509Certificate[] x509Certificates, String s) {
                }

                @Override
                public void checkServerTrusted(X509Certificate[] x509Certificates, String s) {
                }

                @Override
                public X509Certificate[] getAcceptedIssuers() {
                    return null;
                }
            }}, null);

            HttpClient client = HttpClientBuilder.create().disableCookieManagement().
                    disableAutomaticRetries().disableRedirectHandling().
                    disableAuthCaching().setSSLHostnameVerifier(new NoopHostnameVerifier()).setSSLContext(sslContext).build();

            HttpPost post = new HttpPost(url);

            this.generateRequestHeader(post);

            post.setEntity(this.generateRequestBody(requestModal, post));

            for (Header header : post.getAllHeaders()) {
                logger.debug(header.getName() + "," + header.getValue());
            }
            HttpResponse response = client.execute(post);
            return this.convertResponse(handleResponse(response), responseClass);
        } catch (SdkBaseException e) {
            throw e;
        } catch (Exception e) {
            throw new SdkBaseException(e.getMessage());
        }
    }

    public String handleResponse(HttpResponse response) throws IOException,SdkBaseException {
        StatusLine statusLine = response.getStatusLine();
        HttpEntity entity = response.getEntity();
        if (statusLine.getStatusCode() >= 300) {
            ErrorAPIResponse errorAPIResponse = JsonUtils.fromJson(this.handleEntity(entity), ErrorAPIResponse.class);
            throw new SdkBaseException(errorAPIResponse);
        } else {
            return entity == null ? null : this.handleEntity(entity);
        }
    }

    public String handleEntity(final HttpEntity entity) throws IOException{
        return EntityUtils.toString(entity);
    }

    protected void generateRequestHeader(HttpRequestBase request) {
        request.addHeader("contentType", "application/json;charset=UTF-8");
        request.addHeader("x-client-meta", "java-sdk-" + this.profile.getApiVersion());
        request.addHeader("x-ycs-requestid", UUID.randomUUID().toString());
        request.addHeader("x-ycs-timestamp", DateUtils.getISO8601Time(null));
    }

    protected <M extends BaseModel> HttpEntity generateRequestBody(M modal, HttpRequestBase request) throws IOException {
        String requestId = request.getFirstHeader("x-ycs-requestid").getValue();
        String timestamp = request.getFirstHeader("x-ycs-timestamp").getValue();
        String json = JsonUtils.toJson(modal);
        logger.debug(json);
        Map<String, String> params = new HashMap<>();
        params.put("x-ycs-requestid", requestId);
        params.put("x-ycs-timestamp", timestamp);
        params.put("requestBody", json);
        String signString = this.generateSignSummary(params);
        logger.debug("signString is " + signString);
        String authorization = "Authorization: " + this.profile.getSignerName()+" Credential=" + this.credentials.getAppId()+",SignedHeaders=x-ycs-requestid;x-ycs-timestamp,Signature=" + Signers.getSigner(this.profile.getSignerName()).sign(signString, this.credentials.getAppSecret());
        request.addHeader("x-ycs-security-authorization", authorization);
        logger.debug("authorization is : " + authorization);
        return new StringEntity(json, ContentType.APPLICATION_JSON);
    }

    private String generateSignSummary(Map<String, String> summaries) {
        List<String> keys = new ArrayList<>(summaries.keySet());
        Collections.sort(keys);
        StringBuilder sb = new StringBuilder();
        for(String key : keys) {
            sb.append("&").append(key).append("=").append(summaries.get(key));
        }
        return sb.toString().substring(1);
    }

    protected <ResModal extends BaseModel, Res extends BaseAPIResponse<ResModal>> Res convertResponse(String response, Class<Res> responseClass) throws IOException {
        return JsonUtils.fromJson(response, responseClass);
    }

    protected String resolveApiUrl(String uri) {
        Endpoint endpoint = this.profile.getEndpoint();
        if(endpoint == null) endpoint = Endpoints.getGlobalEndpoint();
        if(endpoint == null) throw new NullPointerException("no endpoint configed");
        String url = endpoint.getProtocol() + "://" + endpoint.getEndpoint() + "/" + this.profile.getApiVersion() + uri;
        logger.debug("resolved api url is : " + url);
        return url;
    }
}