package com.yovole.sdk.oss;

import com.fasterxml.jackson.core.type.TypeReference;
import com.yovole.sdk.oss.core.InputSubstream;
import com.yovole.sdk.oss.core.ReleasableInputStream;
import com.yovole.sdk.oss.core.ResettableInputStream;
import com.yovole.sdk.oss.error.OssApiError;
import com.yovole.sdk.oss.exception.SdkClientException;
import com.yovole.sdk.oss.model.*;
import com.yovole.sdk.oss.ssl.TrustAllX509TrustManager;
import com.yovole.sdk.oss.utils.*;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.http.Header;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.*;
import org.apache.http.conn.ssl.NoopHostnameVerifier;
import org.apache.http.entity.ByteArrayEntity;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.FileEntity;
import org.apache.http.entity.InputStreamEntity;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.log4j.Logger;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;
import java.io.*;
import java.net.URLEncoder;
import java.util.*;

import static com.yovole.sdk.oss.model.S3DataSource.Utils.cleanupDataSource;

public class OssApi implements IOssApi {
    //    private RestTemplate restTemplate;
    private String endpoint;
    private String accessKey;
    private String secretKey;
    private String token;
    private Logger logger = Logger.getLogger(getClass());
    private static final String S3_CANNED_ACL = "x-amz-acl";
    public static final String CONTENT_TYPE_JSON = "application/json;charset=UTF-8";
    private static final Integer TIME_OUT = 60 * 1000;
    private static final Integer SOCKET_TIME_OUT = 1800 * 1000;//30分钟
    private static final Set<String> ignoredHeaders = new HashSet<>();
    private static final Integer PART_SIZE = 64 * 1024 * 1024;
    static {
        ignoredHeaders.add(Headers.CONTENT_LENGTH);
        ignoredHeaders.add("X-XSS-Protection");
        ignoredHeaders.add("X-Frame-Options");
        ignoredHeaders.add("Accept-Ranges");
        ignoredHeaders.add(Headers.ETAG);
        ignoredHeaders.add(Headers.METADATA_DIRECTIVE);
        ignoredHeaders.add(Headers.LAST_MODIFIED);
        ignoredHeaders.add("X-Content-Type-Options");
    }

    public OssApi(String endpoint, String accessKey, String secretKey) {
        this.endpoint = endpoint;
        this.accessKey = accessKey;
        this.secretKey = secretKey;
    }

    public OssApi(String endpoint, String token) {
        this.endpoint = endpoint;
        this.token = token;
    }

    /**
     * 设置timeout时间
     *
     * @return
     */
    protected RequestConfig getDefaultRequestConfig() {
        RequestConfig requestConfig = RequestConfig.custom().setConnectionRequestTimeout(TIME_OUT).setConnectTimeout(TIME_OUT).setSocketTimeout(TIME_OUT).build();
        return requestConfig;
    }

    protected RequestConfig getUploadReqeustConfig() {
        RequestConfig requestConfig = RequestConfig.custom().setConnectionRequestTimeout(TIME_OUT).setConnectTimeout(TIME_OUT).setSocketTimeout(SOCKET_TIME_OUT).build();
        return requestConfig;
    }

    public Bucket createBucket(CreateBucketRequest request) {
        String bucketName = request.getBucket();
        bucketName = bucketName.trim();
        BucketNameUtils.validateBucketName(bucketName);
        OssHttpRequest<CreateBucketRequest> httpRequest = new OssHttpRequest<>(HttpMethod.PUT,request);
        if (request.getCannedAcl() != null) addAclHeaders(httpRequest, request.getCannedAcl());
        return doRequest(httpRequest, Bucket.class);
        //return execute(fullPath, HttpMethod.PUT, request, Bucket.class,httpHeaders);
    }


    public Bucket setBucketAcl(SetBucketAclRequest request) {
        request.setSubPath("acl");
        OssHttpRequest<SetBucketAclRequest> httpRequest = new OssHttpRequest<>(HttpMethod.PUT,request);
        addAclHeaders(httpRequest, request.getCannedAcl());
        return doRequest(httpRequest, Bucket.class);
    }

    private void addAclHeaders(OssHttpRequest<?> request, CannedAccessControlList cannedAccessControlList) {
        request.addHeader(S3_CANNED_ACL, cannedAccessControlList.toString());
    }

    public List<Bucket> listBucket(ListBucketRequest request) {
        //String fullPath = getPath("/");
        OssHttpRequest<ListBucketRequest> httpRequest = new OssHttpRequest<>(HttpMethod.GET, request);
        return doRequest(httpRequest, new TypeReference<List<Bucket>>() {
        });
        //return execute(fullPath, HttpMethod.GET, request, new ParameterizedTypeReference<List<Bucket>>() {},prepareHeader(fullPath,HttpMethod.GET));
    }

    public void deleteBucket(DeleteBucketRequest request) {
        OssHttpRequest<DeleteBucketRequest> httpRequest = new OssHttpRequest<>(HttpMethod.DELETE, request);
        doRequestWithoutResponse(httpRequest);
        //execute(fullPath, HttpMethod.DELETE, request, String.class,prepareHeader(fullPath,HttpMethod.DELETE));
    }

    @Override
    public ObjectListing listObject(ListObjectRequest request) throws SdkClientException {
        OssHttpRequest<ListObjectRequest> httpRequest = new OssHttpRequest<>(HttpMethod.GET, request);
        return doRequest(httpRequest, ObjectListing.class);
    }

    @Override
    public String setObjectAcl(SetObjectAclRequest request) throws SdkClientException {
        request.setSubPath("acl");
        OssHttpRequest<SetObjectAclRequest> httpRequest = new OssHttpRequest<>(HttpMethod.PUT, request);
        addAclHeaders(httpRequest, request.getCannedAcl());
        return doRequest(httpRequest, String.class);
    }

    @Override
    public ObjectInfo getObjectInfo(ObjectInfoRequest request) throws SdkClientException {
        request.setSubPath("acl");
        OssHttpRequest<ObjectInfoRequest> httpRequest = new OssHttpRequest<>(HttpMethod.HEAD,request);
        HttpResult result = this.doHttpResult(httpRequest);
        if (result.getStatusCode() >= 200 && result.getStatusCode() < 400) {
//            ObjectInfo objectInfo = new ObjectInfo();
//            objectInfo.setContentType(result.getHeader("Content-Type"));
//            objectInfo.setAcceptRanges(result.getHeader("Accept-Ranges"));
//            objectInfo.seteTag(result.getHeader("ETag").replace("\"", ""));
//            String dateStr = result.getHeader("Last-Modified");
//            try {
//                objectInfo.setLastModified(DateFormatUtils.SMTP_DATETIME_FORMAT.parse(dateStr));
//            } catch (ParseException e) {
//                logger.error("parse Last-Modified date error,str:" + dateStr);
//            }
//            objectInfo.setContentLength(Long.parseLong(result.getHeader("Content-Length")));
//            objectInfo.setCannedAcl(CannedAccessControlList.valueOf(result.getHeader("acl")));
//            return objectInfo;
            ObjectInfo objectInfo = new ObjectInfo();
            objectInfo.setMetadata(MetadataUtils.populateObjectMetadata(result.getResponse(),new ObjectMetadata()));
            objectInfo.setCannedAcl(CannedAccessControlList.valueOf(result.getHeader("acl")));
            return objectInfo;
        } else {
            OssApiError apiError = JacksonUtils.jsonToObj(result.getBody(), OssApiError.class);
            throw new SdkClientException(apiError);
        }
    }

    @Override
    public ObjectMetadata getObjectMetadata(ObjectInfoRequest request) throws SdkClientException {
        OssHttpRequest<ObjectInfoRequest> httpRequest = new OssHttpRequest<>(HttpMethod.HEAD,request);
        HttpResult result = this.doHttpResult(httpRequest);
        if (result.getStatusCode() >= 200 && result.getStatusCode() < 400) {
            return MetadataUtils.populateObjectMetadata(result.getResponse(),new ObjectMetadata());
        } else {
            OssApiError apiError = JacksonUtils.jsonToObj(result.getBody(), OssApiError.class);
            throw new SdkClientException(apiError);
        }
    }

    public void deleteObject(ObjectInfoRequest request) throws SdkClientException {
        OssHttpRequest<ObjectInfoRequest> httpRequest = new OssHttpRequest<>(HttpMethod.DELETE, request);
        doRequestWithoutResponse(httpRequest);
    }

    @Override
    public CannedAccessControlList getObjectAcl(ObjectInfoRequest request) throws SdkClientException {
        request.setSubPath("acl");
        OssHttpRequest<ObjectInfoRequest> httpRequest = new OssHttpRequest<>(HttpMethod.GET,request);
        HttpResult result = this.doHttpResult(httpRequest);
        if (result.getStatusCode() >= 200 && result.getStatusCode() < 400) {
            return CannedAccessControlList.valueOf(result.getHeader("acl"));
        } else {
            OssApiError apiError = JacksonUtils.jsonToObj(result.getBody(), OssApiError.class);
            throw new SdkClientException(apiError);
        }
    }

    protected <T> T doRequestWithHandler(OssHttpRequest httpRequest, BaseResponseHandler<T> handler) throws SdkClientException {
        HttpResponse response = this.doHttpRequest(httpRequest);
        int code = response.getStatusLine().getStatusCode();
        if (code >= 200 && code < 400) {
            return handler.handle(httpRequest,response);
        } else {
            throw SdkClientException.getSdkClientException(HttpUtils.getResponseBody(response));
        }
    }


    protected <T> T doRequest(OssHttpRequest httpRequest, TypeReference typeReference) throws SdkClientException {
        HttpResult result = this.doHttpResult(httpRequest);
        logger.debug(result.getBody());
        if (result.getStatusCode() >= 200 && result.getStatusCode() < 400) {
            return JacksonUtils.jsonToObj(result.getBody(), typeReference, null);
        } else {
            throw SdkClientException.getSdkClientException(result);
        }
    }

    protected <T> T doRequest(OssHttpRequest httpRequest, Class<T> tClass) throws SdkClientException {
        HttpResult result = this.doHttpResult(httpRequest);
        logger.debug(result.getBody());
        if (result.getStatusCode() >= 200 && result.getStatusCode() < 400) {
            if (tClass.getSimpleName().equals("String")) {
                return (T) result.getBody();
            } else {
                return JacksonUtils.jsonToObj(result.getBody(), tClass, null);
            }
        } else {
            throw SdkClientException.getSdkClientException(result);
        }
    }

    private HttpResponse doHttpRequest(OssHttpRequest httpRequest) throws SdkClientException {
        String requestPath = this.endpoint + httpRequest.getPath();
        if (httpRequest.getRequest().getSubPath() != null) {
            requestPath += "?" + httpRequest.getRequest().getSubPath();
        }
        boolean isUpload = false;
        if (httpRequest.getRequest() instanceof S3DataSource) {
            isUpload = true;
        }
        try {
            Iterator<Map.Entry<String, String>> iterator = httpRequest.getParameters().entrySet().iterator();
            String paramStr = "";
            while (iterator.hasNext()) {
                Map.Entry<String, String> entry = iterator.next();
                paramStr += "&" + entry.getKey() + "=" + URLEncoder.encode(entry.getValue(), "UTF-8");
            }
            if (paramStr.length() > 0) {
                if (requestPath.indexOf("?") == -1) {
                    paramStr = paramStr.replaceFirst("&", "?");
                }
                requestPath += paramStr;
            }
            org.apache.http.HttpEntity entity = null;

            ContentType contentType = ContentType.APPLICATION_JSON;
            if (isUpload) {
                contentType = ContentType.APPLICATION_OCTET_STREAM;
                S3DataSource s3DataSource = (S3DataSource) httpRequest.getRequest();

                if (s3DataSource.getFile() != null && s3DataSource.getInputStream() == null) {
                    entity = new InputStreamEntity(new FileInputStream(s3DataSource.getFile()), s3DataSource.getFile().length());
                    contentType = ContentType.create(Mimetypes.getInstance().getMimetype(s3DataSource.getFile().getName()));
                } else {
                    if (s3DataSource instanceof UploadPartRequest) {
                        entity = new InputStreamEntity(s3DataSource.getInputStream(), ((UploadPartRequest) s3DataSource).getPartSize());
                    } else {
                        entity = new InputStreamEntity(s3DataSource.getInputStream());
                    }
                }
            } else if(httpRequest.getMethod().equals(HttpMethod.PUT) || httpRequest.getMethod().equals(HttpMethod.POST)){
                entity = new ByteArrayEntity(JacksonUtils.objToString(httpRequest.getRequest()).getBytes("UTF-8"));
            }
            logger.debug(requestPath);
            HttpRequestBase httpRequestBase;
            if (httpRequest.getMethod().equals(HttpMethod.PUT)) {
                httpRequestBase = new HttpPut(requestPath);
                ((HttpPut) httpRequestBase).setEntity(entity);
            } else if (httpRequest.getMethod().equals(HttpMethod.POST)) {
                httpRequestBase = new HttpPost(requestPath);
                ((HttpPost) httpRequestBase).setEntity(entity);
            } else if (httpRequest.getMethod().equals(HttpMethod.GET)) {
                httpRequestBase = new HttpGet(requestPath);
            } else if (httpRequest.getMethod().equals(HttpMethod.DELETE)) {
                httpRequestBase = new HttpDelete(requestPath);
            } else if (httpRequest.getMethod().equals(HttpMethod.HEAD)) {
                httpRequestBase = new HttpHead(requestPath);
            } else {
                httpRequestBase = new HttpGet(requestPath);
            }
            httpRequest.addAuthHeader(this.accessKey,this.secretKey,this.token);

            httpRequestBase.setConfig(isUpload ? getUploadReqeustConfig() : getDefaultRequestConfig());//set timeout
            Iterator<Map.Entry<String, String>> headerIterator = httpRequest.getHeaders().entrySet().iterator();

            while (headerIterator.hasNext()) {
                Map.Entry<String, String> entry = headerIterator.next();
                httpRequestBase.addHeader(entry.getKey(), entry.getValue());
            }
            if(httpRequestBase.getFirstHeader(Headers.CONTENT_TYPE)==null) {
                httpRequestBase.setHeader(Headers.CONTENT_TYPE, contentType.getMimeType());
            }
            for(Header header : httpRequestBase.getAllHeaders()){
                logger.debug(header.getName() + ": " + header.getValue());
            }
            HttpClient httpClient = getHttpClient();
            return httpClient.execute(httpRequestBase);
        } catch (Exception e) {
            OssApiError apiError = new OssApiError();
            apiError.setMessage(e.getMessage());
            apiError.setStatus(400);
            throw new SdkClientException(apiError);
        }
    }

    protected HttpResult doHttpResult(OssHttpRequest httpRequest) throws SdkClientException {
        return new HttpResult(doHttpRequest(httpRequest));
    }

    protected HttpClient getHttpClient() throws Exception {
        try {
            SSLContext sslContext = SSLContext.getInstance("TLS");
            sslContext.init(null, new TrustManager[]{new TrustAllX509TrustManager()}, null);
            return HttpClientBuilder.create().setSSLContext(sslContext).setSSLHostnameVerifier(new NoopHostnameVerifier()).disableAutomaticRetries().build();
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            throw e;
        }
    }

    protected void doRequestWithoutResponse(OssHttpRequest httpRequest) throws SdkClientException {
        HttpResult result = this.doHttpResult(httpRequest);
        if (result.getStatusCode() >= 400 && result.getStatusCode() < 600) {
            throw SdkClientException.getSdkClientException(result);
        }
    }

    public PutObjectResult putObject(PutObjectRequest request) throws SdkClientException {
        request.validate();
        if (request.getMetadata() == null) request.setMetadata(new ObjectMetadata());
        OssHttpRequest<PutObjectRequest> httpRequest = new OssHttpRequest<>(HttpMethod.PUT, request);
        if (request.getFile() != null) {
            try {
                FileInputStream fis = new FileInputStream(request.getFile());
                if (request.getMetadata().getContentMD5() == null) {
                    byte[] md5 = DigestUtils.md5(fis);
                    String md5Str = Base64.getEncoder().encodeToString(md5);
                    request.getMetadata().setContentMD5(md5Str);
                }
            } catch (Exception e) {
                throw new SdkClientException(e.getMessage(), e);
            }
        }
        populateRequestMetadata(httpRequest, request.getMetadata());
        if (request.getCannedAcl() != null) addAclHeaders(httpRequest, request.getCannedAcl());
        return doRequest(httpRequest, PutObjectResult.class);
    }

    private void populateCopyRequestMetadata(OssHttpRequest<?> request, ObjectMetadata metadata) {
        Map<String, Object> rawMetadata = metadata.getRawMetadata();
        if (rawMetadata != null) {
            for (Map.Entry<String, Object> entry : rawMetadata.entrySet()) {
                if (!ignoredHeaders.contains(entry.getKey())) {
                    request.addHeader(entry.getKey(), entry.getValue().toString());
                }
            }
        }

        Date httpExpiresDate = metadata.getHttpExpiresDate();
        if (httpExpiresDate != null) {
            request.addHeader(Headers.EXPIRES, DateUtils.formatRFC822Date(httpExpiresDate));
        }
        populateUserMetadata(request,metadata.getUserMetadata());
    }


    private void populateRequestMetadata(OssHttpRequest<?> request, ObjectMetadata metadata) {
        Map<String, Object> rawMetadata = metadata.getRawMetadata();
        if (rawMetadata != null) {
            for (Map.Entry<String, Object> entry : rawMetadata.entrySet()) {
                request.addHeader(entry.getKey(), entry.getValue().toString());
            }
        }

        Date httpExpiresDate = metadata.getHttpExpiresDate();
        if (httpExpiresDate != null) {
            request.addHeader(Headers.EXPIRES, DateUtils.formatRFC822Date(httpExpiresDate));
        }
        populateUserMetadata(request,metadata.getUserMetadata());
    }

    private void populateUserMetadata(OssHttpRequest<?> request,Map<String,String> userMetadata){
        if (userMetadata != null) {
            for (Map.Entry<String, String> entry : userMetadata.entrySet()) {
                String key = entry.getKey();
                String value = entry.getValue();
                if (key != null) key = key.trim();
                if (value != null) value = value.trim();
                request.addHeader(Headers.S3_USER_METADATA_PREFIX + key, value);
            }
        }
    }

    @Override
    public DeleteObjectsResult deleteObjects(DeleteObjectsRequest request) throws SdkClientException {
        request.setSubPath("delete");
        OssHttpRequest<DeleteObjectsRequest> httpRequest = new OssHttpRequest<>(HttpMethod.POST, request);
        return doRequest(httpRequest, DeleteObjectsResult.class);
    }

    @Override
    public CopyObjectResult copyObject(CopyObjectRequest request) throws SdkClientException {
        request.validate();
        OssHttpRequest<CopyObjectRequest> httpRequest = new OssHttpRequest<>(HttpMethod.PUT, request);
        httpRequest.addHeader(Headers.COPY_SOURCE, ConvertUtils.getS3Path(request.getSourceBucketName(),request.getSourceKey()));
        if (request.getCannedAccessControlList() != null) {
            httpRequest.addHeader(Headers.S3_CANNED_ACL,
                    request.getCannedAccessControlList().toString());
        }
        if (request.getMetadataDirective() != null) {
            httpRequest.addHeader(Headers.METADATA_DIRECTIVE, request.getMetadataDirective());
        } else if (request.getNewObjectMetadata() != null) {
            httpRequest.addHeader(Headers.METADATA_DIRECTIVE, "REPLACE");
        }
        if (request.getNewObjectMetadata() != null) {
            populateCopyRequestMetadata(httpRequest, request.getNewObjectMetadata());
        }
        return doRequest(httpRequest, CopyObjectResult.class);
    }

    @Override
    public S3Object getObject(String bucketName, String key) throws SdkClientException {
        return getObject(new GetObjectRequest(bucketName, key));
    }


    @Override
    public S3Object getObject(GetObjectRequest request) throws SdkClientException {
        try {
            OssHttpRequest<GetObjectRequest> httpRequest = new OssHttpRequest<>(HttpMethod.GET, request);
            return doRequestWithHandler(httpRequest,new S3ObjectResponseHandler());
        } catch (Exception e){
            throw new SdkClientException(e.getMessage(),e);
        }
    }

    @Override
    public ObjectMetadata getObject(GetObjectRequest getObjectRequest, File destinationFile) throws SdkClientException {
        S3Object object = getObject(getObjectRequest);
        InputStream input = null;
        OutputStream output = null;
        byte[] buffer = new byte[1024];
        try {
            input = object.getObjectContent();
            output = new FileOutputStream(destinationFile);
            for (int length; (length = input.read(buffer)) > 0; ) {
                output.write(buffer, 0, length);
            }
        } catch (Exception e) {
            logger.error(e.getMessage(),e);
            throw new SdkClientException(e.getMessage(),e);
        } finally {
            if (output != null) try { output.close(); } catch (IOException logOrIgnore) {}
            if (input != null) try { input.close(); } catch (IOException logOrIgnore) {}
        }
        return object.getObjectMetadata();
    }

    public PutObjectResult createDir(ObjectInfoRequest request) throws SdkClientException {
        ObjectMetadata dirMetadata = new ObjectMetadata();
        PutObjectRequest putObjectRequest = new PutObjectRequest(request.getBucket(), request.getKey(), new ByteArrayInputStream(new byte[0]), dirMetadata).withRequest(request);
        return putObject(putObjectRequest);
    }

    /**
     * 只设置主信息，保留userMetaData信息,主信息6项覆盖
     * @param request
     * @return CopyObjectResult
     * @throws SdkClientException
     */
    @Override
    public CopyObjectResult setObjectMainMetadata(SetObjectMainMetadataRequest request) throws SdkClientException {
        request.validate();
        ObjectInfoRequest objectInfoRequest = new ObjectInfoRequest(request.getBucket(),request.getKey()).withRequest(request);
        ObjectMetadata metadata = getObjectMetadata(objectInfoRequest);
        metadata.setHttpExpiresDate(request.getNewMetadata().getHttpExpiresDate());
        Map<String, Object> rawMetadata = request.getNewMetadata().getRawMetadata();
        if (rawMetadata != null) {
            for (Map.Entry<String, Object> entry : rawMetadata.entrySet()) {
                metadata.setHeader(entry.getKey(), entry.getValue().toString());
            }
        }
        return setObjectMetadata(request,metadata);
    }

    /**
     * 只设置userData信息，保留main信息,userMetadata信息覆盖
     * @param request
     * @return CopyObjectResult
     * @throws SdkClientException
     */
    @Override
    public CopyObjectResult setObjectUserMetadata(SetObjectUserMetadataRequest request) throws SdkClientException {
        request.validate();
        ObjectInfoRequest objectInfoRequest = new ObjectInfoRequest(request.getBucket(),request.getKey()).withRequest(request);
        ObjectMetadata metadata = getObjectMetadata(objectInfoRequest);
        metadata.getUserMetadata().clear();
        metadata.getUserMetadata().putAll(request.getUserMetadata());
        return setObjectMetadata(request,metadata);
    }

    private CopyObjectResult setObjectMetadata(BaseRequest request,ObjectMetadata newMetadata){
        CopyObjectRequest copyObjectRequest = new CopyObjectRequest(request.getBucket(), request.getKey(),
                request.getBucket(), request.getKey()).withRequest(request);
        if (newMetadata != null) {
            copyObjectRequest.setNewObjectMetadata(newMetadata);
        }
        return copyObject(copyObjectRequest);
    }

    @Override
    public CopyObjectResult setObjectMetadata(SetObjectMetadataRequest request) throws SdkClientException {
        request.validate();
        ObjectInfoRequest objectInfoRequest = new ObjectInfoRequest(request.getBucket(),request.getKey()).withRequest(request);
        ObjectMetadata metadata = getObjectMetadata(objectInfoRequest);
        metadata.setHttpExpiresDate(request.getNewMetadata().getHttpExpiresDate());
        Map<String, Object> rawMetadata = request.getNewMetadata().getRawMetadata();
        if (rawMetadata != null) {
            for (Map.Entry<String, Object> entry : rawMetadata.entrySet()) {
                metadata.setHeader(entry.getKey(), entry.getValue().toString());
            }
        }
        metadata.getUserMetadata().clear();
        metadata.getUserMetadata().putAll(request.getNewMetadata().getUserMetadata());
        return setObjectMetadata(request,metadata);
    }

    @Override
    public InitiateMultipartUploadResult initiateMultipartUpload(
            InitiateMultipartUploadRequest initiateMultipartUploadRequest)
            throws SdkClientException {
        initiateMultipartUploadRequest.validate();
        initiateMultipartUploadRequest.setSubPath("uploads");
        if (initiateMultipartUploadRequest.getObjectMetadata() == null){
            initiateMultipartUploadRequest.setObjectMetadata(new ObjectMetadata());
            initiateMultipartUploadRequest.getObjectMetadata().setContentType(Mimetypes.getInstance().getMimetype(initiateMultipartUploadRequest.getKey()));
        }
        OssHttpRequest<InitiateMultipartUploadRequest> request = new OssHttpRequest<>(HttpMethod.POST,initiateMultipartUploadRequest);
        if (initiateMultipartUploadRequest.getCannedAcl() != null) addAclHeaders(request, initiateMultipartUploadRequest.getCannedAcl());
        populateRequestMetadata(request, initiateMultipartUploadRequest.objectMetadata);
        return doRequest(request, InitiateMultipartUploadResult.class);
    }

//    private List<PartETag> uploadParts(String bucket, String object,
//                                       String uploadId, File file) {
//        List<PartETag> partETags = new ArrayList<>();
//        long contentLength = file.length();
//        long partSize = PART_SIZE;
//        try {
//            long compSize = 0;
//            for (int i = 1; compSize < contentLength; i++) {
//                partSize = Math.min(partSize, (contentLength - compSize));
//                UploadPartRequest uploadRequest = new UploadPartRequest()
//                        .withBucketName(bucket)
//                        .withKey(object)
//                        .withUploadId(uploadId).withPartNumber(i)
//                        .withFileOffset(compSize).withFile(file).withPartSize(partSize);
//                partETags.add(uploadPart(uploadRequest).getPartETag());
//                compSize += partSize;
//            }
//        } catch (Exception e) {
//            logger.error(e.getMessage(), e);
//            abortMultipartUpload(new AbortMultipartUploadRequest(bucket, object, uploadId));
//        }
//        return partETags;
//    }

    public UploadPartResult uploadPart(UploadPartRequest uploadPartRequest) throws SdkClientException {
        uploadPartRequest.validate();
        final File fileOrig = uploadPartRequest.getFile();
        final InputStream isOrig = uploadPartRequest.getInputStream();
        final String uploadId   = uploadPartRequest.getUploadId();
        final int partNumber    = uploadPartRequest.getPartNumber();
        final long partSize     = uploadPartRequest.getPartSize();

        OssHttpRequest<UploadPartRequest> request = new OssHttpRequest<>(HttpMethod.PUT, uploadPartRequest);
        request.addParameter("uploadId", uploadId);
        request.addParameter("partNumber", Integer.toString(partNumber));

        ObjectMetadata objectMetadata = uploadPartRequest.getObjectMetadata();
//        if (uploadPartRequest.getFile() != null) {
//            try {
//                if(objectMetadata == null) objectMetadata = new ObjectMetadata();
//                if (objectMetadata.getContentMD5() == null) {
//                    FileInputStream fis = new FileInputStream(uploadPartRequest.getFile());
//                    byte[] md5 = DigestUtils.md5(fis);
//                    String md5Str = Base64.getEncoder().encodeToString(md5);
//                    objectMetadata.setContentMD5(md5Str);
//                }
//            } catch (Exception e) {
//                throw new SdkClientException(e.getMessage(), e);
//            }
//        }

        if (objectMetadata != null)
            populateRequestMetadata(request, objectMetadata);
        addHeaderIfNotNull(request, Headers.CONTENT_MD5, uploadPartRequest.getMd5Digest());
        //request.addHeader(Headers.CONTENT_LENGTH, Long.toString(partSize));

        InputStream isCurr = isOrig;
        try {
            if (fileOrig == null) {
                if (isOrig == null) {
                    throw new IllegalArgumentException(
                            "A File or InputStream must be specified when uploading part");
                } else {
                    // When isCurr is a FileInputStream, this wrapping enables
                    // unlimited mark-and-reset
                    isCurr = ReleasableInputStream.wrap(isCurr);
                }
            } else {
                try {
                    isCurr = new ResettableInputStream(fileOrig);
                } catch(IOException e) {
                    throw new IllegalArgumentException("Failed to open file "
                            + fileOrig, e);
                }
            }
            isCurr = new InputSubstream(
                    isCurr,
                    uploadPartRequest.getFileOffset(),
                    partSize,
                    uploadPartRequest.isLastPart());
            request.getRequest().setInputStream(isCurr);
            return doRequest(request, UploadPartResult.class);
        } finally {
            cleanupDataSource(uploadPartRequest, fileOrig, isOrig, isCurr);
        }
    }

    private static void addHeaderIfNotNull(OssHttpRequest<?> request, String header, String value) {
        if (value != null) {
            request.addHeader(header, value);
        }
    }

    public void abortMultipartUpload(AbortMultipartUploadRequest request) throws SdkClientException {
        OssHttpRequest<AbortMultipartUploadRequest> httpRequest = new OssHttpRequest<>(HttpMethod.DELETE, request);
        doRequestWithoutResponse(httpRequest);
    }

//    public boolean uploadBigObject(UploadBigFileRequest request) {
//        request.validate();
//        if(request.getMetadata()==null) request.setMetadata(new ObjectMetadata());
//        if (request.getFile() != null) {
//            try {
//                if (request.getMetadata().getContentMD5() == null) {
//                    request.getMetadata().setContentMD5(getMd5Str(request.getFile()));
//                }
//            } catch (Exception e) {
//                throw new SdkClientException(e.getMessage(), e);
//            }
//        }
//        InitiateMultipartUploadRequest initUploadRequest = new InitiateMultipartUploadRequest(request.getBucket(), request.getKey());
//        InitiateMultipartUploadResult initResult = initiateMultipartUpload(initUploadRequest);
//        String uploadId = initResult.getUploadId();
//        try {
//            List<PartETag> partETags = uploadParts(request.getBucket(), request.getKey(), uploadId, request.getFile());
//            CompleteMultipartUploadRequest request1 = new CompleteMultipartUploadRequest(request.getBucket(), request.getKey(),uploadId,partETags);
//            completeMultiPartUpload(request1);
//        } catch (Exception e) {
//            logger.error(e.getMessage(),e);
//            abortMultiObject(request.getBucket(), request.getKey(), uploadId);
//            return false;
//        }
//        return true;
//    }

//    private String getMd5Str(File file) {
//        try {
//            FileInputStream fis = new FileInputStream(file);
//            byte[] md5 = DigestUtils.md5(fis);
//            return Base64.getEncoder().encodeToString(md5);
//        } catch (Exception e) {
//            throw new SdkClientException(e.getMessage(), e);
//        }
//    }

    public CompleteMultipartUploadResult completeMultiPartUpload(CompleteMultipartUploadRequest request) throws SdkClientException{
        OssHttpRequest<CompleteMultipartUploadRequest> httpRequest = new OssHttpRequest<>(HttpMethod.POST, request);
        httpRequest.addParameter("uploadId",request.getUploadId());
        return doRequest(httpRequest, CompleteMultipartUploadResult.class);
    }

//    private void abortMultiObject(String bucket, String object, String uploadId) {
//        logger.debug(String.format("DELETE /{bucket}/{object}?uploadId=  Input: %s %s %s", bucket, object, uploadId));
//        try {
////            s3client.abortMultipartUpload(new AbortMultipartUploadRequest(
////                    bucket, object, uploadId));
//        } catch (Exception e) {
//            logger.error(e.getMessage(),e);
//        }
//    }

    @Override
    public MultipartUploadListing listMultipartUploads(ListMultipartUploadsRequest request) throws SdkClientException {
        request.validate();
        request.setSubPath("uploads");
        OssHttpRequest<ListMultipartUploadsRequest> httpRequest = new OssHttpRequest<>(HttpMethod.GET, request);
        return doRequest(httpRequest, MultipartUploadListing.class);
    }

    @Override
    public PartListing listParts(ListPartsRequest request) throws SdkClientException {
        request.validate();
        OssHttpRequest<ListPartsRequest> httpRequest = new OssHttpRequest<>(HttpMethod.GET, request);
        return doRequest(httpRequest, PartListing.class);
    }

    @Override
    public void enableCustomer(EnableOrDisableCustomerRequest request) throws SdkClientException {
        OssHttpRequest<EnableOrDisableCustomerRequest> httpRequest = new OssHttpRequest<>(HttpMethod.POST, "/enableCustomer",request);
        doRequestWithoutResponse(httpRequest);
    }

    @Override
    public void disableCustomer(EnableOrDisableCustomerRequest request) throws SdkClientException {
        OssHttpRequest<EnableOrDisableCustomerRequest> httpRequest = new OssHttpRequest<>(HttpMethod.POST, "/disableCustomer",request);
        doRequestWithoutResponse(httpRequest);
    }

}
