package com.yovole.sdk.oss.core;

import com.yovole.sdk.oss.exception.SdkClientException;
import java.io.IOException;
import java.io.InputStream;

public final class InputSubstream extends SdkFilterInputStream {
    private static final int MAX_SKIPS = 100;
    private long currentPosition;
    private final long requestedOffset;
    private final long requestedLength;
    private final boolean closeSourceStream;
    private long markedPosition = 0;

    public InputSubstream(InputStream in, long offset, long length,
                          boolean closeSourceStream) {
        super(in);

        this.currentPosition = 0;
        this.requestedLength = length;
        this.requestedOffset = offset;
        this.closeSourceStream = closeSourceStream;
    }

    @Override
    public int read() throws IOException {
        byte[] b = new byte[1];
        int bytesRead = read(b, 0, 1);

        if (bytesRead == -1)
            return bytesRead;
        return b[0];
    }

    @Override
    public int read(byte[] b, int off, int len) throws IOException {
        int count = 0;
        while (currentPosition < requestedOffset) {
            long skippedBytes = super.skip(requestedOffset - currentPosition);
            if (skippedBytes == 0) {
                count++;
                if (count > MAX_SKIPS) {
                    throw new SdkClientException(
                            "Unable to position the currentPosition from "
                                    + currentPosition + " to "
                                    + requestedOffset);
                }
            }
            currentPosition += skippedBytes;
        }

        long bytesRemaining =
            (requestedLength + requestedOffset) - currentPosition;
        if (bytesRemaining <= 0)
            return -1;

        len = (int) Math.min(len, bytesRemaining);
        int bytesRead = super.read(b, off, len);
        currentPosition += bytesRead;

        return bytesRead;
    }

    @Override
    public synchronized void mark(int readlimit) {
        markedPosition = currentPosition;
        super.mark(readlimit);
    }

    @Override
    public synchronized void reset() throws IOException {
        currentPosition = markedPosition;
        super.reset();
    }

    @Override
    public void close() throws IOException {
        if (closeSourceStream)
            super.close();
    }

    @Override
    public int available() throws IOException {
        long bytesRemaining;
        if (currentPosition < requestedOffset)
            bytesRemaining = requestedLength;
        else {
            bytesRemaining =
                (requestedLength + requestedOffset) - currentPosition;
        }

        return (int) Math.min(bytesRemaining, super.available());
    }

    InputStream getWrappedInputStream() {
        return in;
    }
}
