package com.yovole.sdk.oss.core;

import com.yovole.sdk.oss.exception.SdkClientException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.channels.FileChannel;

public class ResettableInputStream extends ReleasableInputStream {
    private static final Log log = LogFactory
            .getLog(ResettableInputStream.class);
    private final File file; // null if the file is not known
    private FileInputStream fis; // never null
    private FileChannel fileChannel; // never null

    private long markPos;

    public ResettableInputStream(File file) throws IOException {
        this(new FileInputStream(file), file);
    }

    public ResettableInputStream(FileInputStream fis) throws IOException {
        this(fis, null);
    }

    /**
     * @param file
     *            can be null if not known
     */
    private ResettableInputStream(FileInputStream fis, File file) throws IOException {
        super(fis);
        this.file = file;
        this.fis = fis;
        this.fileChannel = fis.getChannel();
        this.markPos = fileChannel.position();
    }

    @Override
    public final boolean markSupported() {
        return true;
    }

    @Override
    public void mark(int _) {
        abortIfNeeded();
        try {
            markPos = fileChannel.position();
        } catch (IOException e) {
            throw new SdkClientException("Failed to mark the file position", e);
        }
        if (log.isTraceEnabled())
            log.trace("File input stream marked at position " + markPos);
    }

    @Override
    public void reset() throws IOException {
        abortIfNeeded();
        fileChannel.position(markPos);
        if (log.isTraceEnabled())
            log.trace("Reset to position " + markPos);
    }

    @Override
    public int available() throws IOException {
        abortIfNeeded();
        return fis.available();
    }

    @Override
    public int read() throws IOException {
        abortIfNeeded();
        return fis.read();
    }

    @Override
    public long skip(long n) throws IOException {
        abortIfNeeded();
        return fis.skip(n);
    }

    @Override
    public int read(byte[] arg0, int arg1, int arg2) throws IOException {
        abortIfNeeded();
        return fis.read(arg0, arg1, arg2);
    }

    public File getFile() {
        return file;
    }

    public static ResettableInputStream newResettableInputStream(File file) {
        return newResettableInputStream(file, null);
    }

    public static ResettableInputStream newResettableInputStream(File file,
            String errmsg) {
        try {
            return new ResettableInputStream(file);
        } catch (IOException e) {
            throw errmsg == null
                ? new SdkClientException(e)
                : new SdkClientException(errmsg, e);
        }
    }

    public static ResettableInputStream newResettableInputStream(
            FileInputStream fis) {
        return newResettableInputStream(fis, null);
    }

    public static ResettableInputStream newResettableInputStream(
            FileInputStream fis, String errmsg) {
        try {
            return new ResettableInputStream(fis);
        } catch (IOException e) {
            throw new SdkClientException(errmsg, e);
        }
    }
}
