package com.yovole.sdk.oss.model;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.yovole.sdk.oss.auth.SignerConstants;
import com.yovole.sdk.oss.utils.ConvertUtils;
import org.apache.commons.lang3.StringUtils;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import java.io.InputStream;
import java.time.OffsetDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.*;

public class OssHttpRequest<T extends BaseRequest> {
    private HttpMethod method;
    private String path;
    private T request;
    private Map<String, String> headers = new HashMap<>();
    private Map<String, String> parameters = new LinkedHashMap<>();
    private InputStream content;
    private static final String API_VERSION = "2018-10-30";

    public OssHttpRequest(HttpMethod method, T request) {
        String path = ConvertUtils.getS3Path(request.getBucket(),request.getKey());
        initParams(method,path,request);
    }

    public OssHttpRequest(HttpMethod method, String path, T request) {
        initParams(method,path,request);
    }

    private void initParams(HttpMethod method, String path, T request){
        this.method = method;
        this.path = path;
        request.validate();
        this.request = request;
        if (this.request != null) {
            if (method == HttpMethod.GET || method == HttpMethod.DELETE) {
                ObjectMapper oMapper = new ObjectMapper();
                this.setParameters(oMapper.convertValue(request, new TypeReference<Map<String, String>>() {
                }));
            }
        }
    }

    public HttpMethod getMethod() {
        return method;
    }

    public String getPath() {
        return path;
    }

    public void addParameter(String name, String value) {
        parameters.put(name, value);
    }

    public void setParameters(Map<String, String> parameters) {
        this.parameters.clear();
        this.parameters.putAll(parameters);
    }

    public Map<String, String> getParameters() {
        return parameters;
    }

    public T getRequest() {
        return request;
    }

    public void setRequest(T request) {
        this.request = request;
    }

    public Map<String, String> getHeaders() {
        return headers;
    }

    public void setHeaders(Map<String, String> headers) {
        this.headers.clear();
        this.headers.putAll(headers);
    }

    public void addHeader(String key, String value) {
        this.headers.put(key, value);
    }

//    public String getAuthPath(){
//        if ("policy".equals(this.getSubPath())) {
//            return this.path += "?" + this.getSubPath();
//        } else {
//            return this.path;
//        }
//    }

    public InputStream getContent() {
        return content;
    }

    public void setContent(InputStream content) {
        this.content = content;
    }

    public void addAuthHeader(String accessKey,String secretKey,String token) {
        if(StringUtils.isNotEmpty(token)) {
            this.addHeader(SignerConstants.AUTHORIZATION, token);
            this.addHeader(Headers.IAM_VPP, this.getRequest().getVppId());
            this.addHeader(Headers.IAM_CUSTOMER, this.getRequest().getCustomerId());
            this.addHeader(Headers.IAM_USER, this.getRequest().getUserId());
        } else {
            OffsetDateTime dateTime = OffsetDateTime.now(ZoneId.of("GMT"));
            DateTimeFormatter formatter = DateTimeFormatter.RFC_1123_DATE_TIME;
            String date = dateTime.format(formatter);
            Map<String, String> authHeaders = new HashMap<>();
            authHeaders.put(SignerConstants.DATE, date);
            authHeaders.put(SignerConstants.NONCE, UUID.randomUUID().toString());
            authHeaders.put(SignerConstants.VERSION, API_VERSION);

            final String credential = "Credential=" + accessKey;
            final String signerHeaders = "SignedHeaders=" + getSignedHeadersString(authHeaders);
            final String signature = "Signature=" + base64Sha1Hmac(getCanonicalizedHeaderString(authHeaders), secretKey);
            authHeaders.put(SignerConstants.AUTHORIZATION, createStringToSign(credential, signerHeaders, signature));
            this.getHeaders().putAll(authHeaders);
        }
    }

    protected String createStringToSign(String credential,String signerHeaders,String signature) {
        final StringBuilder authHeaderBuilder = new StringBuilder();
        authHeaderBuilder.append(SignerConstants.YV_SIGNING_ALGORITHM)
                .append(" ")
                .append(credential)
                .append(", ")
                .append(signerHeaders)
                .append(", ")
                .append(signature);
        return authHeaderBuilder.toString();
    }

    protected String getCanonicalizedHeaderString(Map<String,String> authHeader) {
        final List<String> sortedHeaders = new ArrayList<String>(authHeader
                .keySet());
        Collections.sort(sortedHeaders, String.CASE_INSENSITIVE_ORDER);

        StringBuilder buffer = new StringBuilder();
        for (String header : sortedHeaders) {
            String key = StringUtils.lowerCase(header);
            String value = authHeader.get(header);
            buffer.append(key);
            buffer.append(":");
            if (value != null) {
                buffer.append(value);
            }
            buffer.append("\n");
        }
        return buffer.toString();
    }

    protected String getSignedHeadersString(Map<String,String> authHeader) {
        final List<String> sortedHeaders = new ArrayList<String>(authHeader.keySet());
        Collections.sort(sortedHeaders, String.CASE_INSENSITIVE_ORDER);

        StringBuilder buffer = new StringBuilder();
        for (String header : sortedHeaders) {
            if (buffer.length() > 0)
                buffer.append(";");
            buffer.append(StringUtils.lowerCase(header));
        }

        return buffer.toString();
    }

    /*
     * Takes a uri and a secret key and returns a base64-encoded
     * SHA-1 HMAC.
     */
    private String base64Sha1Hmac(String uri, String secretKey) {
        try {
            byte[] keyBytes = secretKey.getBytes("UTF-8");
            SecretKeySpec signingKey = new SecretKeySpec(keyBytes, "HmacSHA1");

            Mac mac = Mac.getInstance("HmacSHA1");
            mac.init(signingKey);

            byte[] rawHmac = mac.doFinal(uri.getBytes("UTF-8"));

            Base64.Encoder base64 = Base64.getEncoder();
            return base64.encodeToString(rawHmac);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
}
